/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package examples.invoice.queries;

import examples.invoice.applications.AddressAppli;
import examples.invoice.applications.JormInit;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.comp.cmdline.api.CmdLineParser;
import org.objectweb.jorm.comp.cmdline.rdb.RdbMapperOption;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.comp.compiler.lib.BasicCompiler;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Schema;
import org.objectweb.util.io.api.PathExplorer;
import org.objectweb.util.io.lib.DirJavaExplorer;
import org.objectweb.medor.query.jorm.lib.JormExtent;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.api.Field;

import java.io.FileInputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.Properties;
import javax.resource.ResourceException;
import javax.resource.cci.ConnectionFactory;

/**
 *
 * @author	<A HREF="mailto:alia.mourad@rd.francetelecom.com><b>
 *			Mourad Alia
 *			</b></A>
 *		<A HREF="mailto:alexandre.lefebvre@rd.francetelecom.com><b>
 * 			Alexandre Lefebvre
 </b></A>
 */
public class AddressQuery extends AddressAppli {

	private static AddressQuery singleQuery =
		new AddressQuery();

	public static void main(String[] args) {
		if (args.length != 1) {
			System.err.println("Usage: AddressQuery prop_file_name");
			System.exit(-1);
		}
		singleQuery.runAddressQuery(args[0]);
	}

	/**
	 * @param propfilename is the filename for the runtime.properties file in the
	 * etc directory of the invoice example in the JORM dist.
	 */
	public void runAddressQuery(String addPropFile) {
		Properties addProp = new Properties();
		try {
			addProp.load(new FileInputStream(addPropFile));
			String propfilename = addProp.getProperty("runtime", null);
			//links to the properties file
			Properties prop = new Properties();
			System.out.println(propfilename);
			prop.load(new FileInputStream(propfilename));
			JormInit.getInstance().initLogSystem(prop);
			JormInit.getInstance().initResourceAdapter(prop);
			prop = null;

			mapClasses();
			if (activeMapper == null) {
				System.out.println("null active mapper");
			}
			else {
				System.out.println("Active mapper is " + activeMapper.getMapperName());
			}

			//Compiles the source pd file
			//Initilization of the compiler
			BasicCompiler comp = new BasicCompiler();
			CompilerParameter compPar = comp.getCompilerParameter();
			PathExplorer pathex = new DirJavaExplorer();
			pathex.addPath(addProp.getProperty("pddir", null));
			compPar.setClasspath(pathex);
			compPar.setLogConfFile(addProp.getProperty("logconf", null));

			compPar.setMapperName("rdb");
			compPar.setMappingFactory("org.objectweb.jorm.metainfo.lib.rdb.BasicRDBMappingFactory");
			compPar.setMappingName("INVOICE_RDB");
			compPar.setParserName("org.objectweb.jorm.parser.lib.rdb.RDBParser");
			compPar.setVerifierName("org.objectweb.jorm.verifier.lib.rdb.RdbVerifier");

			String pddir = addProp.getProperty("pddir");
			compPar.addInputFileName(pddir + java.io.File.separator + "Address.pd");
			/*
			compPar.addInputFileName(pddir + java.io.File.separator + "Invoice.pd");
			compPar.addInputFileName(pddir + java.io.File.separator + "Product.pd");
			compPar.addInputFileName(pddir + java.io.File.separator + "ProdId.pd");
			compPar.addInputFileName(pddir + java.io.File.separator + "PUId.pd");
			compPar.addInputFileName(pddir + java.io.File.separator + "ProdUnits.pd");
			*/
			RdbMapperOption rop = new RdbMapperOption();
			//need to get the logger.
			rop.setLogger(JormInit.getInstance().getLoggerFactory().getLogger("examples.invoice.applications.JormInit"));
			((CmdLineParser) rop).parseProperty("DBName", "postgres");
			compPar.addCPExtension(
				"org.objectweb.jorm.comp.cmdline.rdb.RdbMapperOption",
				rop);

			System.out.println("Compiler initialized");

			//call the parser
			Collection metaobjs = comp.parseFiles(compPar.getInputFileNames());

			System.out.println("Pasing finished");

			Iterator metaIt = metaobjs.iterator();

			while (metaIt.hasNext()) {
				MetaObject mo = (MetaObject) metaIt.next();
				if (mo instanceof Class) {
					System.out.println("Initializing class name " +
										((Class) mo).getName());
					String fullClass =
						((Schema) ((Class) mo).getParent()).getName() + "." +
						((Class) mo).getName();
					PClassMapping pm =
						activeMapper.lookup(fullClass);
					//add the meta objects from the parsing
					pm.init(activeMapper, mo);
				}
			}
			//build a JORM extent for class Address
            PClassMapping pm =
				activeMapper.lookup("examples.invoice.persistclass.Address");
			QueryLeaf ql = new JormExtent(((Class)pm.getMetaInfo()),
				activeMapper, "Address", false);
			System.out.println("Created new extent (all fields)");

			//looking at field names
			Field[] fields = ql.getTupleStructure().getFields();
			for (int i = 0; i < fields.length; i++) {
				System.out.println("Field name "+fields[i].getName());
			}

			String[] selected = {"name", "zip"};
			QueryLeaf q2 = new JormExtent(((Class)pm.getMetaInfo()),
						activeMapper, selected, true, "Address");
			System.out.println("Created new extent (selected fields)");
			//looking at field names
			Field[] fields2 = q2.getTupleStructure().getFields();
			for (int i = 0; i < fields2.length; i++) {
				System.out.println("Field name "+fields2[i].getName());
			}

		}
		catch (Exception e) {
			System.err.println("Exit with exception: " + e.getMessage());
			do {
				e.printStackTrace();
				if (e instanceof PException)
					e = ((PException) e).getNestedException();
				if (e instanceof ResourceException)
					e = ((ResourceException) e).getLinkedException();
				else
					e = null;
			} while (e != null);
		}

	}

	private void mapClasses() {
		String name = "jdbc.default";
		PMapper m;
		ConnectionFactory cf;

		m = JormInit.getInstance().getMapper(name, null);
		activeMapper = m;
		if (m == null) {
			errorMsg("no associated mapper");
			return;
		}
		if (m.getMapperName().equals("rdb")) {
			cf = JormInit.getInstance().getJdbcCF(name, null, null, null);
		}
		else if (m.getMapperName().equals("fos")) {
			cf = JormInit.getInstance().getFosCF(name, null);
		}
		else {
			errorMsg("unsupported mapper type");
			return;
		}
		if (cf == null) {
			errorMsg("no associated connection factory");
			return;
		}

		if (JormInit.getInstance().isMapped(name)) {
			errorMsg("map has already been performed");
			return;
		}
		if (!JormInit.getInstance().canMap(name)) {
			errorMsg("cannot map with this mapper");
			return;
		}
		try {
			getHelper().map(m, cf, JormInit.getInstance().getLoggerFactory());
			JormInit.getInstance().setMapped(name);
			System.out.println("Persistent classes mapped.");
		}
		catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}

}
