/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.datasource.lib;

import org.objectweb.medor.api.DataSourceException;
import org.objectweb.medor.datasource.api.DataStore;

public class BasicDataStore implements DataStore {

	String sourceName;
	String clientName;
    short[] capabilities;
    short type;

    /**
     * The capacities are restrected...
     */
    public BasicDataStore(short dataStoreType,
						  String sourceName,
						  short[] capabilities)
            throws DataSourceException {
        this.sourceName = sourceName;
        this.capabilities = capabilities;
        if (dataStoreType != DataStore.JDBC_STORE &&
			dataStoreType != DataStore.MEDORTC_STORE)
            throw new DataSourceException("DataStore type not supported");
        this.type = dataStoreType;
    }

    /**
     * Full capacities evaluation...
     */
    public BasicDataStore(short dataStoreType, String sourceName)
            throws DataSourceException {
        this.sourceName = sourceName;
        if (dataStoreType != DataStore.JDBC_STORE &&
			dataStoreType != DataStore.MEDORTC_STORE)
            throw new DataSourceException("DataStore type not supported");
        this.type = dataStoreType;
    }

    public String getName() {
        return sourceName;
    }

    /**
     * A simple example of an implementation of this method will tests equality
     * between DataStore name only. Others implemntation will be done later.
     */
    public boolean isSameAs(DataStore ds) {
        return ds.getName().equals(this.sourceName);
    }

    public boolean isCapable(short operation) {
        if (capabilities == null)
            return true;// Full capabilities
        else {
            int cpt;
            for (cpt = 0; (cpt < capabilities.length &&
							capabilities[cpt] != operation); cpt++) ;
            if (cpt == capabilities.length)
                return false;
            else
                return true;
        }
    }

    public short getDataStoreType() {
        return type;
    }

	public String getClientName() {
		return clientName;
	}

	public void setClientName(String cName) {
		clientName = cName;
	}
}
