/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.eval.api;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;

/**
 * This interface defines the controller object for the evaluation of a given
 * QueryNode within a QueryTree. It is created at evaluation time by the
 * NodeEvaluatorFactory, on request from the QueryEvaluator.
 * The QueryEvaluator controls the overall evaluation by activating/killing
 * the NodeEvaluator instances.
 */
public interface NodeEvaluator {
	/**
	 * Closes the links between the current NodeEvaluator and the NodeEvaluators
	 * of the children QueryTrees for pipelining operations.
	 * TODO complete comments
	 */
	boolean unlinkChildren();

	/**
	 * Opens the links between the current NodeEvaluator and the NodeEvaluators
	 * of the children QueryTrees for pipelining operations.
	 * TODO complete comments
	 */
	boolean linkChildren();

	/**
	 * This method executes the query on the underlying node.
	 * It controls the evaluation locally for this node.
	 * It returns the TupleCollection containing the result of the execution.
	 */
	TupleCollection fetchData(ParameterOperand[] parameters)
		throws MedorException;

    /**
     * Associates a PrefetchBuffer to be filled by the collection computed by
     * this evaluator.
     * @param pb   The PrefetchBuffer to be filled.
     */
    void setPrefetchBuffer(PrefetchBuffer pb) throws MedorException;

    long getCacheSize();

    void setCacheSize(long cacheSize);

	/**
	 * Returns the evaluation meta data associated with this node evaluator.
	 * @return the associated EvaluationMetaData
	 */
	EvaluationMetaData getEvaluationMetaData();
    /**
     * Returns the QueryNode to which the NodeEvaluator is attached.
     */
	QueryNode getQueryNode();
}
