/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.eval.cache.lib;

/**
 * This class implements a basic TupleCollection wich is cached.
 */

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.eval.cache.api.CachedTupleCollection;
import org.objectweb.medor.eval.cache.api.CollectionCache;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;

public class WindowCachedTupleCollection implements CachedTupleCollection {

	private boolean closed = false;
    public WindowCachedTupleCollection(TupleCollection tc,
									   CollectionCache tupleCache)
		throws MedorException {
        this.tupleCache = tupleCache;
        this.tc = tc;
    }

    private CollectionCache tupleCache;
    private TupleCollection tc;
    private int cursor = 1;

	public void close() throws MedorException {
		closed =true;
	}

    public TupleStructure getMetaData() throws MedorException{
        return tc.getMetaData();
    }

    public CollectionCache getCache() {
        return tupleCache;
    }

    public boolean isLast() throws MedorException {
        if (tc.getRow() == cursor) {
            return tc.isLast();
        } else
            return (!tupleCache.contains(cursor + 1));
    }

    public boolean next() throws MedorException {
        if (!isLast()) {
            cursor++;
            if (!tupleCache.contains(cursor + 1)) tc.next();
            return true;
        } else
            return false;
    }

    public void first() throws MedorException {
        cursor = 1;
        tupleCache.initialize();
        tc.first();
    }

    public Tuple getTuple() throws MedorException {
        Tuple t = null;
        t = tupleCache.getTuple(cursor);
        if (t == null) {
            //System.out.println("Not From Cache memory ");
            //TODO use cloning process, to be implemented by Tuple
			//t = (Tuple) tc.getTuple().clone();
			t = (Tuple) tc.getTuple();
            tupleCache.putTuple(cursor, t);
            if (tc.isLast()) {
                tupleCache.setCanInsert(false);
            }
        } else {
            //System.out.println("from cache memory");
        }
        return t;
    }

    public Tuple getTuple(int row) throws MedorException {
        Tuple t = null;
        t = tupleCache.getTuple(row);
        if (t == null) {
            //System.out.println("Not From Cache memory ");
			//TODO use cloning process, to be implemented by Tuple
			//t = (Tuple) tc.getTuple().clone();
            t = (Tuple)tc.getTuple(row);
            tupleCache.putTuple(row, t);
            if (tc.isLast()) {
                tupleCache.setCanInsert(false);
            }
        } //isGetFromCache = true;
        return t;
    }

    public boolean row(int numTuple) throws MedorException {
        //isGetFromCache = false;
        cursor = numTuple;
        tupleCache.initialize();
        return (tc.row(numTuple));
    }

    public int getRow() throws MedorException {
        return cursor;
    }

    public boolean isEmpty() throws MedorException{
        return tc.isEmpty();
    }

    /**
     * This method is used when nesting data. It must be carrefully implemented
     */
    public int getLeftTCCursor() {
        return 0;
    }

    /**
     * This method is used when nesting data. It must be carrefully implemented
     */
    public int getRightTCCursor() {
        return 0;
    }

}
