/**
 * Copyright (C) 2001-2006 France Telecom R&D
 */
package org.objectweb.medor.eval.lib;

import org.objectweb.medor.api.EvaluationException;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.datasource.api.WrapperFactory;
import org.objectweb.medor.datasource.lib.MedorWrapperFactory;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.eval.api.QueryEvaluator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.VariableOperand;
import org.objectweb.medor.lib.BasicField;
import org.objectweb.medor.lib.BasicTupleStructure;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.tuple.lib.ExplicitTupleCollection;
import org.objectweb.medor.tuple.lib.MemoryTuple;

import java.util.Map;

public abstract class AbstractQueryEvaluator implements QueryEvaluator {

    /**
     * The query tree which must be evaluated
     */
    protected QueryTree query;

    /**
     * The factory of wrappers (leaf evaluator)
     */
    protected WrapperFactory wrapperFactory;

    
    public AbstractQueryEvaluator(QueryTree query)
            throws EvaluationException {
        this.query = query;
        wrapperFactory = new MedorWrapperFactory();
    }


    // IMPLEMENTATION OF THE QueryEvaluator INTERFACE //
    //------------------------------------------------//

    public WrapperFactory getWrapperFactory() {
        return wrapperFactory;
    }

    public QueryTree getQueryTree() {
        return query;
    }

    public static TupleCollection getMemoryTupleCollection(VariableOperand[] values) 
        throws MedorException {
        Field[] fs = new Field[values.length];
        for (int i = 0; i < values.length; i++) {
            fs[i] = new BasicField("field_" + i, values[i].getType());
        }
        ExplicitTupleCollection tc = new ExplicitTupleCollection(new BasicTupleStructure(fs));
        tc.insertTuple(new MemoryTuple(values));
        return tc;
    }
}
