/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.eval.lib;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.medor.eval.api.EvaluationMetaData;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class BasicEvaluationMetaData implements EvaluationMetaData {

	private Map annotations = new HashMap();
	
	public BasicEvaluationMetaData() {
		annotations.put(EvaluationMetaData.ANNOTATION_IS_CACHE, Boolean.FALSE);
		annotations.put(EvaluationMetaData.ANNOTATION_IS_PARALLEL, Boolean.FALSE);
		annotations.put(EvaluationMetaData.ANNOTATION_LIMITED_RANGE_START_AT, new Integer(0));
		annotations.put(EvaluationMetaData.ANNOTATION_LIMITED_RANGE_SIZE, new Integer(Integer.MAX_VALUE));
	}

	public void setCache(boolean isCache) {
		annotations.put(EvaluationMetaData.ANNOTATION_IS_CACHE, new Boolean(isCache));
	}

	public boolean isCache() {
	    Boolean isCache = (Boolean)annotations.get(EvaluationMetaData.ANNOTATION_IS_CACHE);
		return isCache.booleanValue();
		}

	public void setParallel(boolean isParallel) {
		annotations.put(EvaluationMetaData.ANNOTATION_IS_PARALLEL, new Boolean(isParallel));
	}

	public boolean isParallel() {
	    Boolean isParallel = (Boolean)annotations.get(EvaluationMetaData.ANNOTATION_IS_PARALLEL);
		return isParallel.booleanValue(); 
	}

	public boolean isLimitedRange() {
	  return ((Integer)annotations.get(EvaluationMetaData.ANNOTATION_LIMITED_RANGE_SIZE)).intValue() < Integer.MAX_VALUE;
	}

	public void setLimitedRangeStartAt(int start) {
	    annotations.put(EvaluationMetaData.ANNOTATION_LIMITED_RANGE_START_AT, new Integer(start));
	}

	public int getLimitedRangeStartAt() {
	    return ((Integer)annotations.get(EvaluationMetaData.ANNOTATION_LIMITED_RANGE_START_AT)).intValue();
	}

	public void setLimitedRangeSize(int number) {
	    annotations.put(EvaluationMetaData.ANNOTATION_LIMITED_RANGE_SIZE, new Integer(number));
	}

	public int getLimitedRangeSize() {
	    return ((Integer)annotations.get(EvaluationMetaData.ANNOTATION_LIMITED_RANGE_SIZE)).intValue();
	}
	
	public Map getAnnotations() {
	    return annotations;
	}
}
