/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.eval.lib;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.api.NodeEvaluator;
import org.objectweb.medor.eval.api.NodeEvaluatorFactory;
import org.objectweb.medor.eval.api.EvaluationMetaData;
import org.objectweb.medor.query.api.QueryNode;

/**
 * This class manage creation the iteratifs NodeEvaluators corresponding to
 * QueryNodes. The method createNodeEvaluator is able to generate a
 * NodeEvaluator for a Binary QueryNode and for Unary QueryNode, otherwise it
 * throws a MedorException
 */
public class IteratifNodeEvaluatorFactory implements NodeEvaluatorFactory {

	public NodeEvaluator createNodeEvaluator(QueryNode node,
											 NodeEvaluator[] subEvaluators,
											 EvaluationMetaData evalMD)
			throws MedorException {
		if ((node.getChildren().length == 1) || (subEvaluators.length == 1)) {
			return new UnaryIteratifNodeEvaluator(node,
					subEvaluators[0],
					evalMD);
		} else if ((node.getChildren().length == 2) ||
				(subEvaluators.length == 2)) {
			return new BinaryIteratifNodeEvaluator(node,
					subEvaluators[0],
					subEvaluators[1],
					evalMD);
		} else
			throw new MedorException("This QueryNode is not evaluable: too many children (" + node.getChildren().length + ").");
	}
}
