/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.eval.lib;

import org.objectweb.medor.eval.api.NodeEvaluator;
import org.objectweb.medor.eval.api.EvaluationMetaData;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.api.MedorException;

/**
 *  Just a simple class that encapsulates a TupleCollection.
 */
public class TupleCollectionNodeEvaluator implements NodeEvaluator {

	TupleCollection tc;
	QueryNode qn;
	long cacheSize = 0;

    public TupleCollectionNodeEvaluator(TupleCollection tc) {
        this.tc = tc;
    }

    public boolean linkChildren(){
        return true;
    }

    public boolean unlinkChildren(){
        return true;
    }

    public TupleCollection fetchData(ParameterOperand[] parameters){
        return tc;
    }

    public QueryNode getQueryNode(){
        return qn;
    }

    public long getCacheSize() {
        return cacheSize;
    }

    public void setCacheSize(long cacheSize) {
        this.cacheSize = cacheSize;
    }

    public void setPrefetchBuffer(PrefetchBuffer pb) throws MedorException {
        // No prefetching for this evaluator: ignore
    }

	public EvaluationMetaData getEvaluationMetaData() {
		return null;
	}

}
