/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.medor.eval.prefetch.api;

/**
 * It acts as an intermediary prefetch buffer.
 * It has a delegate prefetch buffer 
 * and a translation table between the Tuple of the delegate prefetch buffer 
 * and its own Tuple.
 * @author Y.Bersihand
 */
public interface IntermediaryPrefetchBuffer extends PrefetchBuffer {
	
	/**
	 * Sets the delegate prefetch buffer
	 */
	void setDelegatePrefetchBuffer(PrefetchBuffer pb);
	
	/**
	 * Gets the delegate prefetch buffer
	 * @return
	 */
	PrefetchBuffer getDelegatePrefetchBuffer();
	
	/**
	 * Sets the association table between the tuple of the delegate prefetch buffer 
	 * and its tuple.
	 * Example: Tuple of the delegate pb : [f1, f2, f3, f4]
	 * 			association table : [0,3] because to build its tuple,
	 * 			only the fields f1 and f4 are needed.
	 * @param indexes
	 */
	void setAssociationTable(int[] indexes);
	
	/**
	 * Gets the association table.
	 */
	int[] getAssociationTable();
	
}
