/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.medor.eval.prefetch.lib;

import org.objectweb.jorm.mapper.rdb.lib.RdbIntermediaryTuple;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.IntermediaryPrefetchBuffer;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;

/**
 * @author Y.Bersihand
 */
public class IntermediaryPrefetchBufferImpl implements
		IntermediaryPrefetchBuffer {

	private PrefetchBuffer delegatePb;
	private int[] associationTable;
	
	public IntermediaryPrefetchBufferImpl(PrefetchBuffer delegatePb, int[] associationTable) {
		this.delegatePb = delegatePb;
		this.associationTable = associationTable;
	}

	public void setDelegatePrefetchBuffer(PrefetchBuffer pb) {
		this.delegatePb = pb;
	}

	public PrefetchBuffer getDelegatePrefetchBuffer() {
		return delegatePb;
	}

	public void setAssociationTable(int[] indexes) {
		associationTable = indexes;
	}

	public int[] getAssociationTable() {
		return associationTable;
	}

	//just delegates the process
	public void addPrefetchTuple() throws MedorException {
		delegatePb.addPrefetchTuple();
	}

	//just delegates the process
	public void setTupleCollection(TupleCollection tc) throws MedorException {
		delegatePb.setTupleCollection(tc);
	}

	//just delegates the process
	public void close() throws MedorException {
		delegatePb.close();
	}

	//just delegates the process
	public boolean isClosed() {
		return delegatePb.isClosed();
	}

	/**
	 * Uses the association table to change the tuple retrieved from the delegatePb
	 * into a valid tuple for this prefetch buffer.
	 * If no delegate pb tuple found, return null.
	 */
	public Tuple getTuple(Object index) throws MedorException {
		Tuple delegatePbTuple = delegatePb.getTuple(index);
		//if no delegate pb tuple, return null
		if(delegatePbTuple == null)
			return null;
		RdbIntermediaryTuple intermediaryTuple = new RdbIntermediaryTuple(associationTable, delegatePbTuple);
		return intermediaryTuple;
	}

	public TupleCollection getTupleCollection(Object index) throws MedorException {
		return delegatePb.getTupleCollection(index);
	}

}
