/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.medor.filter.api.AggregateOperator;
import org.objectweb.medor.expression.lib.BasicUnaryOperator;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.jorm.type.api.PType;

import java.util.Map;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicAggregateOperator
	extends BasicUnaryOperator
	implements AggregateOperator {

    protected boolean distinct = false;

	public BasicAggregateOperator() {
	}

	public BasicAggregateOperator(Expression e) {
		super(e);
	}

	public BasicAggregateOperator(PType type) {
		super(type);
	}

	public BasicAggregateOperator(PType type, Expression e) {
		super(type, e);
	}

	public BasicAggregateOperator(Expression e, boolean distinct) {
		super(e);
		this.distinct = distinct;
	}

	public BasicAggregateOperator(PType type, Expression e, boolean distinct) {
		super(type, e);
		this.distinct = distinct;
	}

	public void setDistinct(boolean distinct) {
		this.distinct = distinct;
	}

	public boolean getDistinct() {
		return distinct;
	}

	public Object clone(Object clone, Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((BasicAggregateOperator) clone).distinct = distinct;
		return clone;
	}
}
