/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.UnaryOperator;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.lib.BasicUnaryOperator;
import org.objectweb.medor.expression.lib.BasicVariableOperand;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.type.lib.QTypeTuple;

/**
 * IsEmpty operates on an Expression which must be of a TupleCollection type, or
 * a GenClassRef. Its evaluation returns true if the Expression is an empty
 * TupleCollection or if the GenClassRef has no element.
 */
public class IsEmpty extends BasicUnaryOperator implements UnaryOperator {

    private static final long serialVersionUID = 4200517014796518079L;

    public IsEmpty() {
        super(PTypeSpace.BOOLEAN);
    }

    public IsEmpty(Expression e) {
        super(PTypeSpace.BOOLEAN, e);
    }

    public Operand evaluate(ParameterOperand[] pos, Object o)
            throws ExpressionException {
        Operand subResult = expressions[0].evaluate(pos, o);
        switch (subResult.getType().getTypeCode()) {
        case QTypeTuple.TYPECODE_REFERENCE:
            // TODO: raffiner pour les genclassref
            result.setValue(subResult.getObject());
            break;
        case QTypeTuple.TYPECODE_TUPLE_COLLECTION:
            TupleCollection tc = (TupleCollection) subResult.getObject();
            try {
                result.setValue(tc.isEmpty());
            } catch (MedorException e) {
                throw new ExpressionException(
                        "Unevaluable Expression: Not compiled");
            }
            break;
        default:
            throw new MalformedExpressionException(
                    "Unauthorized expression element");
        }
        return result;
    }

    public Operand compileExpression() throws ExpressionException,
            MalformedExpressionException {
        if (expressions[0] != null) {
            expressions[0].compileExpression();
            // TODO raffiner pour les genclassref
            if ((expressions[0].getType().getTypeCode() == QTypeTuple.TYPECODE_REFERENCE)
                    || (expressions[0].getType().getTypeCode() == QTypeTuple.TYPECODE_TUPLE_COLLECTION)) {
                result = new BasicVariableOperand(type);
                verified = true;
            } else {
                throw new TypingException(
                        "IsEmpty only operates on GenClassReferences and TupleCollections");
            }
        } else
            // Operands not yet assigned
            throw new MalformedExpressionException("null children value");
        return result;
    }

    public String getOperatorString() {
        return "IsEmpty";
    }
}
