/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.rdb.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.filter.rdb.api.RdbFieldOperand;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicRdbField;

/*
 * @author	<A HREF="mailto:alia.mourad@rd.francetelecom.com><b>
 *			Mourad Alia
 *			</b></A>
 *		<A HREF="mailto:alexandre.lefebvre@rd.francetelecom.com><b>
 * 			Alexandre Lefebvre
			</b></A>
 */

public class BasicRdbFieldOperand
	extends BasicRdbField
		implements RdbFieldOperand {

    BasicRdbFieldOperand(String name,
                         PType type,
                         String col,
                         QueryLeaf ql) {
        super(name, type, col, ql);
    }

    /**
     * Can be considered as compiled, since nothing to do.
     * */
    public boolean isCompiled() {
        return true;
    }

    /**
     * nothing to do.
     */
    public Operand compileExpression()
            throws TypingException, MalformedExpressionException {
		return null;
    }

    /**
     * No result (this is not evaluable).
     */
    public Operand getResult() throws IllegalStateException {
        throw new IllegalStateException("An RdbFieldOperand cannot be evaluatead");
    }

    /**
     * Not evaluable.
     */
    public Operand evaluate(ParameterOperand[] values, Object o)
            throws ExpressionException {
        throw new ExpressionException("An RdbFieldOperand cannot be evaluatead");
    }
}
