/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.medor.optim.rdb;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.Like;
import org.objectweb.medor.filter.rdb.lib.Like2SQLStringConverter;
import org.objectweb.medor.optim.lib.BasicRule;
import org.objectweb.medor.query.api.FilteredQueryTree;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.util.monolog.api.BasicLevel;

public class Like2SQL extends BasicRule {

    public Like2SQL() {
        super();
    }

    public Like2SQL(String suffix) {
        super(suffix);
    }

    public QueryTree rewrite(QueryTree qt, QueryNode _parent)
            throws MedorException {
        if (qt instanceof FilteredQueryTree) {
            FilteredQueryTree fqt = (FilteredQueryTree) qt;
            ModifiedExpression me = convertRegExpToSQLLike(
                    fqt.getQueryFilter(), false);
            if (me.isModified)
                fqt.setQueryFilter(me.e);
        }
        if (qt instanceof QueryNode) {
            QueryTree[] children = ((QueryNode) qt).getChildren();
            for (int i = 0; i < children.length; i++) {
                rewrite(children[i], (QueryNode) qt);
            }
        }
        return qt;
    }
    private ModifiedExpression convertRegExpToSQLLike(Expression e, boolean inLike) throws MedorException {
        ModifiedExpression me = new ModifiedExpression(e, false);
        if (inLike) {
            if (e instanceof Like2SQLStringConverter) {
            } else if (e instanceof BasicParameterOperand) {
                me.e = new Like2SQLStringConverter((BasicParameterOperand) e); 
                me.isModified = true;
            } else if (e.getClass().equals(BasicOperand.class)) {
                me.isModified = true;
                try {
                    me.e = new BasicOperand(
                            Like2SQLStringConverter.convert(
                                    ((BasicOperand) e).getString()));
                } catch (TypingException e1) {
                    throw new MedorException("Expected a basicOperand with String type", e1);
                }
            }
        } else if (e instanceof Operator) {
        	if (!inLike) {
        		inLike = e instanceof Like;
        	}
            Operator op = (Operator) e;
            for (int i = 0; i < op.getOperandNumber(); i++) {
                ModifiedExpression mei = convertRegExpToSQLLike(op.getExpression(i), inLike);
                if (mei.isModified) {
                    op.setExpression(i, mei.e);
                }
            }
        }
        return me; 
    }
}
