/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.api;

import org.objectweb.medor.clone.api.Cloneable;

/**
 * Represents an element of an "order by" clause, qualified by a field and
 * a boolean indicating whether the order is descendant (false, default)
 * or ascendant.
 * @author Alexandre Lefebvre
 */
public interface OrderField extends Cloneable{

	/**
	 * Sets the field of the clause part.
	 * @param field the field by which to order.
	 */
	void setField(QueryTreeField field);

	/**
	 * Obtains the field by which to order.
	 * @return the field by which to order.
	 */
	QueryTreeField getField();

	/**
	 * Sets the descendant order.
	 * <p>The default value is false.
	 * @param desc indicates whether the order is descendant (true) or
	 * ascendant (false).
	 */
	void setDesc(boolean desc);

	/**
	 * Returns whether the order is descendant (true) or ascendant (false).
	 * @return true if the order is descendant, false if the order is ascendant.
	 */
	boolean getDesc();
}

