/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.api;

import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.api.QueryNodeException;
import org.objectweb.medor.expression.api.Expression;

/**
 * This interface is used for creating QueryNodes.
 */
public interface QueryNodeFactory {

	/**
	 * This methods creates the links between the children QueryTrees and
	 * the created QueryNode. It is used when building a QueryNode from existing
	 * QueryTrees.
	 * It typically verifies that all Fields defined in the tupleStructure are
	 * present in one of the children.
	 * It also propagates the indexes in the associated TupleLoaders for
	 * evaluation purposes.
	 */
	QueryNode createQueryNode(QueryTree[] children, short OperationType,
							  Expression filter, TupleStructure tupleStructure)
		throws QueryNodeException;

}
