/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.api;

import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.clone.api.Cloneable;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface QueryTree extends Cloneable, AnnotationsHolder {

	/**
	 * Returns the TupleStructure associated to the QueryTree.
	 * Note that the Fields cannot be added by manipulating the TupleStructure,
	 * but by using either constructors in the case of a QueryLeaf, or by
	 * using the addXXXFields methods in the case of QueryNodes.
	 */
	TupleStructure getTupleStructure();

    /**
     * It returns the name of the query tree.
     */
    String getName();

	/**
	 * Tests whether the results should be distinct or not.
	 * @return true if there results should be distinct, false otherwise.
	 */
	boolean getDistinct();

	/**
	 * Indicates whether evaluating the QueryTree should return unique results
	 * (duplicate elimination) or not.
	 * <p>The default should be that duplicates are not eliminated (distinct
	 * is false).
	 * @param distinct is true if duplicates should be eliminated, false
	 * otherwise.
	 * @throws MedorException if the distinct operation is not supported.
	 */
	void setDistinct(boolean distinct) throws MedorException;

	/**
	 * Sets the OrderBy array of OrderFields.
	 * <p>This array indicates the fields by which the results should be
	 * ordered.
	 * <p>Like for filters assigned to QueryNodes, the Fields used for ordering
	 * must belong to children of the current QueryTree.
	 * <p>A noticeable exception is the case of RdbExpQueryLeaves, where
	 * ordering fields are fields of the QueryLeaf itself.
	 * @param orders the array of OrderFields indicating the ordering in which
	 * the result should be returned.
	 * @throws MedorException if ordering is not supported.
	 */
	void setOrderBy(OrderField[] orders) throws MedorException;

	/**
	 * Returns the array of OrderFields indicating how the result should be
	 * ordered.
	 * @return the array of OrderFields.
	 */
	OrderField[] getOrderBy();
}
