/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.query.lib;

import org.objectweb.medor.query.api.OrderField;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.clone.lib.BasicCloneable;

import java.util.Map;

/**
 * @author Alexandre Lefebvre
 */
public class BasicOrderField extends BasicCloneable implements OrderField {

	private boolean desc = false;

	private QueryTreeField field;

	public BasicOrderField() {
	}

	/**
	 * Constructs a BasicOrderField from a QueryTreeField.
	 * <p>The default value for ordering is ascendant.
	 * @param field the QueryTreeField on which to build the ordering.
	 */
	public BasicOrderField(QueryTreeField field) {
		this.field = field;
	}

	/**
	 * Constructs a BasciOrderField from a QueryTreeField, with indication
	 * of ordering.
	 * @param field the QueryTreeField on which to build the ordering
	 * @param desc the boolean indicating whether the ordering should be
	 * descendant (true) or ascendant (false).
	 */
	public BasicOrderField(QueryTreeField field, boolean desc) {
		this.field = field;
		this.desc = desc;
	}

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((BasicOrderField) clone).desc = desc;
		((BasicOrderField) clone).field = (QueryTreeField) getClone(field, obj2clone);
		return clone;
	}

	public void setField(QueryTreeField field) {
		this.field = field;
	}

	public QueryTreeField getField() {
		return field;
	}

	public void setDesc(boolean desc) {
		this.desc = desc;
	}

	public boolean getDesc() {
		return desc;
	}
}
