/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.query.lib;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.lib.BasicTupleStructure;
import org.objectweb.medor.query.api.OrderField;
import org.objectweb.medor.query.api.QueryTree;

import java.util.HashMap;
import java.util.Map;

/**
 * A BasicQueryNode is an implementation of QueryNode.
 * It also implements QueryIndexGenerator and RWTupleStructure (it is its own
 * TupleStructure).
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicQueryTree extends BasicTupleStructure
	implements QueryTree {

    protected String name = null;
	/**
	 * Contains ranges of attribute to be projected from the tuple resulting
	 * of the concatenation of the tuples from the children QueryTrees.
	 */
	protected int[] indexes;

	protected boolean distinct = false;

	protected OrderField[] orderfields = null;

	protected Map annotations = new HashMap();
	
	public BasicQueryTree() {
	}

	public BasicQueryTree(String _name) {
		super();
        this.name = _name;
	}

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		BasicQueryTree bqt = (BasicQueryTree) clone;
		bqt.name = name;
		bqt.distinct = distinct;
		if (indexes != null) {
			bqt.indexes = new int[indexes.length];
			System.arraycopy(indexes, 0, bqt.indexes, 0, indexes.length);
		}
		if (orderfields != null) {
			bqt.orderfields = new OrderField[orderfields.length];
			for(int i=0; i<orderfields.length; i++) {
				bqt.orderfields[i] = (OrderField) getClone(orderfields[i],obj2clone);
			}
		}
		return clone;
	}

    public String getName() {
        return name;
    }

	public TupleStructure getTupleStructure() {
		return this;
	}

	public void setDistinct(boolean distinct) throws MedorException {
		this.distinct = distinct;
	}

	public boolean getDistinct() {
		return distinct;
	}

	public void setOrderBy(OrderField[] orderfields) {
		this.orderfields = orderfields;
	}

	public OrderField[] getOrderBy() {
		return orderfields;
	}
	
	public Map getAnnotations() {
	    return annotations;
	}
}
