/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.lib;

import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.query.api.OperationType;
import org.objectweb.medor.query.api.QueryTree;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class JoinProject extends BasicQueryNode {

	private boolean outer = true;

	public JoinProject() {
	}

	public JoinProject(boolean outer) {
		this.outer = outer;
	}

	/**
	 * Basic constructor. Fields and filter must be added later.
	 */
	public JoinProject(String name) {
		super(name);
	}

	public JoinProject(String name, boolean outer) {
		super(name);
		this.outer = outer;
	}

	/**
	 * Basic constructor. Fields must be added later.
	 */
        public JoinProject(Expression filter, String name) {
                super(name);
                this.setQueryFilter(filter);
        }

	public JoinProject(Expression filter, String name, boolean outer) {
                super(name);
                this.setQueryFilter(filter);
		this.outer = outer;
	}

        public short getType() {
                return OperationType.JOIN_PROJECT;
        }

	protected void updateChildren() {
		super.updateChildren();
		for (int i = 0; i < children.size(); i++) {
			setOuter((QueryTree) children.get(i), outer);
		}
	}

}
