/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2005 France Telecom
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.query.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.BasicDataStore;
import org.objectweb.medor.lib.BasicTupleStructure;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.api.TCQueryLeaf;
import org.objectweb.medor.query.api.OrderField;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.tuple.api.TupleLoader;
import org.objectweb.medor.expression.api.Expression;

import java.util.HashMap;
import java.util.Map;

/**
 * A TupleCollection is one of the DataStore that is queryable by Medor.
 * <p>This
 * class reprensents a QueryLeaf wich take its data from a Medor
 * TupleCollection.
 */
public class MedorTCQueryLeaf extends BasicTupleStructure
		implements TCQueryLeaf {

	protected TupleCollection tcData;
	protected TupleLoader loader;
	protected DataStore ds;
	protected Expression filter;
	protected String nodeName;

	protected Map annotations = new HashMap();

	public MedorTCQueryLeaf() {
	}

	public MedorTCQueryLeaf(String nodeName, String tcName,
							TupleCollection tcData)
			throws MedorException {
		super();
		this.nodeName = nodeName;
		this.tcData = tcData;
		ds = new BasicDataStore(DataStore.MEDORTC_STORE, tcName);
	}

	public QueryTreeField addField(String fieldName, PType type, Field tcField)
			throws MedorException {
		QueryTreeField qtf =
				new BasicQueryTreeField(
						getFieldName(nodeName, fieldName), type, this);
		name2field.put(qtf.getName(), qtf);
		fields.add(qtf);
		return qtf;
	}

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		MedorTCQueryLeaf mql = (MedorTCQueryLeaf) clone;
		mql.nodeName = nodeName;
		mql.tcData = tcData;
		mql.ds = ds;
		mql.loader = loader;
		mql.filter = (Expression) getClone(filter, obj2clone);
		return clone;
	}

	public String getName() {
		return nodeName;
	}

	public TupleStructure getTupleStructure() {
		return this;
	}

	public void setDistinct(boolean d) throws MedorException {
		throw new MedorException("Distinct not yet supported on MedorTCQueryLeaf");
	}

	public boolean getDistinct() {
		return false;
	}

	public void setOrderBy(OrderField[] orderfields) throws MedorException {
		throw new MedorException("Order by not supported on MedorTCQueryLeaf");
	}

	public OrderField[] getOrderBy() {
		return null;
	}

	public DataStore getDataStore() {
		return ds;
	}

	public TupleCollection getTupleCollection() {
		return tcData;
	}

	public TupleLoader getTupleLoader() {
		return loader;
	}

	public void setTupleLoader(TupleLoader tl) {
		this.loader = tl;
	}

	public Expression getQueryFilter() {
		return filter;
	}

	public void setQueryFilter(Expression filter) {
		this.filter = filter;
	}

	private String getFieldName(String nodeName, String fieldName) {
		if (nodeName == null || nodeName.length() == 0)
			return fieldName;
		else
			return nodeName + "." + fieldName;
	}

	public Map getAnnotations() {
	    return annotations;
	}
}
