/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.rdb.api;

import org.objectweb.medor.query.api.QueryLeaf;

/**
 * An RdbExpField is a particular type of RdbField, which is associated to
 * an RdbExpQueryLeaf, and which refers to a QualifiedTable.
 * @see org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf
 *
 * @author	<A HREF="mailto:alia.mourad@rd.francetelecom.com><b>
 *			Mourad Alia
 *			</b></A>
 *		<A HREF="mailto:alexandre.lefebvre@rd.francetelecom.com><b>
 *			Alexandre Lefebvre
 * 			</b></A>
 */
public interface RdbExpField extends RdbField {
	/**
	 * Returns the associated QualifiedTable.
	 * @return the associated QualifiedTable in the underlying SQL query.
	 */
	QualifiedTable getTable();

	/**
	 * Associates the RdbExpField with the QualifiedTable.
	 * @param t is the associated QualifiedTable in the underlying SQL query.
	 */
	void setTable(QualifiedTable t);

	/**
	 * Sets the QueryLeaf to which the RdbExpField is associated
	 * @param ql is the QueryLeaf to which the RdbExpField is associated
	 */
	void setQueryLeaf(QueryLeaf ql);

}
