/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.rdb.api;

import org.objectweb.jorm.type.api.PType;

/**
 * An RdbStringQueryLeaf is an RdbQueryLeaf built from a SQL request provided
 * as a String.
 * The Medor optimizer cannot "reason" with an RdbStringQueryLeaf, and namely
 * cannot collapse several RdbStringQueryLeaves on the same datastore into one.
 * Such an RdbStringQueryLeaf is thus a "black box".
 * If RdbQueryLeaves should be collapsed into one for optimisation purposes,
 * RdbExpQueryLeaves should be used instead.
 * @see org.objectweb.medor.query.rdb.api.RdbQueryLeaf
 * @see org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf
 *
 * @author	<A HREF="mailto:alia.mourad@rd.francetelecom.com><b>
 *			Mourad Alia
 *			</b></A>
 *		<A HREF="mailto:alexandre.lefebvre@rd.francetelecom.com><b>
 *			Alexandre Lefebvre
 * 			</b></A>
 */
public interface RdbStringQueryLeaf extends RdbQueryLeaf {
	/**
	 * Adds a Field to the RdbExpQueryLeaf
	 * @param fieldName is the name of the Field to be added.
	 * @param type is the PTyme of the Field to be added.
	 * @param nameInQuery is the name of the column in the associated
	 * QualifiedTable.
	 * @return the created RdbExpField.
     */
	RdbField addRdbField(String fieldName,
						 PType type,
						 String nameInQuery);

}
