/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.query.rdb.lib;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;

public class BasicQualifiedTable extends JoinedTable implements QualifiedTable {

	private boolean polymorphic;
	private boolean withSubclasses;
	
	/*
	 * Used to define groups of tables, for example when transforming JormExtents
	 * into relational tables.
	 */
	private Map annotations = new HashMap();

	public BasicQualifiedTable() {
		super();
		polymorphic = false;
		withSubclasses = false;
	}
	
    /**
     * Builds a BasicQualifiedTable from a table name an an alias name.
     */
    public BasicQualifiedTable(String tableName, String aliasName, 
            boolean polymorphic, boolean withSubclasses) {
        super(tableName, aliasName);
        this.polymorphic = polymorphic;
        this.withSubclasses = withSubclasses;
    }

    public BasicQualifiedTable(String tableName, String aliasName) {
        super(tableName, aliasName);
        this.polymorphic = false;
    }
    
    public String getTableName() {
        return tableName;
    }

    public String getAliasName() {
        return alias;
    }

    public void setAliasName(String aliasName) {
        this.alias = aliasName;
    }
    
	public boolean isPolymorphic() {
		return polymorphic;
	}
	
	public boolean withSubclasses() {
		return withSubclasses;
	}
	
	public Map getAnnotations() {
	    return annotations;
	}
}

