/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.rdb.lib;

import java.util.ArrayList;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.QueryLeafException;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.query.api.FilteredQueryTree;
import org.objectweb.medor.query.rdb.api.RdbField;
import org.objectweb.medor.query.rdb.api.RdbStringQueryLeaf;

/**
 *
 * This class represents a QueryLeaf that maps onto a relational database store.
 */
public class BasicRdbStringQueryLeaf extends BasicRdbQueryLeaf
	implements RdbStringQueryLeaf, FilteredQueryTree {

	public BasicRdbStringQueryLeaf() {
	}

	/**
	 * Constructs a BasicRdbStringQueryLeaf from an SQL string.
	 */
	public BasicRdbStringQueryLeaf(DataStore ds,
								   String query,
								   String nodeName) {
		super(nodeName, ds, query);
	}

	public RdbField addRdbField(String name, PType type, String colName) {
		RdbField f = new BasicRdbField(name, type, colName, this);
		name2field.put(f.getName(), f);
		fields.add(f);
		return f;
	}

	public String getSqlRequest(ParameterOperand[] pos,
	        boolean rangeStartAt,
	        boolean rangeSize) {
		return query;
	}

	public String getSqlRequest(ParameterOperand[] pos,
								ArrayList al,
								boolean rangeStartAt,
								boolean rangeSize) throws MedorException {
		if (al == null) {
			return getSqlRequest(pos, rangeStartAt, rangeSize);
		} else {
			throw new MedorException("Request with selectList not supported");
		}
	}

    public String getSqlDelete(ParameterOperand[] pos) {
        // TODO Auto-generated method stub
        return null;
    }

    public String getSqlUpdate(ParameterOperand[] pos, Expression updateExpression) {
        // TODO Auto-generated method stub
        return null;
    }

	public String getSelectList(String selectList,
								ArrayList selectFields,
								boolean qualified) throws MedorException {
		throw new QueryLeafException("Unsupported method getSelectList for BasicRdbStringQueryLeaf");
	}

	public Expression getQueryFilter() {
		return sqlFilter;
	}

	public void setQueryFilter(Expression exp) {
		this.sqlFilter = exp;
	}	
}
