/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.tuple.lib;

import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.clone.lib.BasicCloneable;
import org.objectweb.medor.api.MedorException;

import java.util.Map;

/**
 * This class represents an empty TupleCollection.
 *
 * @author S.Chassande-Barrioz
 */
public class EmptyTupleCollection
	extends BasicCloneable
	implements TupleCollection {

    private TupleStructure ts;
	private boolean closed;

	public EmptyTupleCollection() {
	}

    public EmptyTupleCollection(TupleStructure _ts) {
        ts = _ts;
		closed =false;
    }

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((EmptyTupleCollection) clone).closed = closed;
		((EmptyTupleCollection) clone).ts = (TupleStructure) getClone(ts, obj2clone);
		return clone;
	}

    public TupleStructure getMetaData() throws MedorException {
		if (closed) {
			throw new MedorException("Impossible to use a closed TupleCollection");
		}
        return ts;
    }

    public boolean isLast() throws MedorException {
		if (closed) {
			throw new MedorException("Impossible to use a closed TupleCollection");
		}
        return false;
    }

    public boolean next() throws MedorException {
		if (closed) {
			throw new MedorException("Impossible to use a closed TupleCollection");
		}
        return false;
    }

    public void first() throws MedorException {
        throw new MedorException("Empty result");
    }

    public int getRow() throws MedorException {
        throw new MedorException("Empty result");
    }

    public Tuple getTuple() throws MedorException {
        throw new MedorException("Empty result");
    }

    public Tuple getTuple(int row) throws MedorException {
        throw new MedorException("Empty result");
    }

    public boolean row(int row) throws MedorException {
        throw new MedorException("Empty result");
    }

    public boolean isEmpty() throws MedorException {
		if (closed) {
			throw new MedorException("Impossible to use a closed TupleCollection");
		}
        return true;
    }

	public void close() throws MedorException {
		closed = true;
	}
}
