/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.expression.api.BinaryOperator;
import org.objectweb.medor.expression.api.Comparator;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.Minus;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.expression.lib.Greater;
import org.objectweb.medor.expression.lib.Substring;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.lib.BasicField;
import org.objectweb.medor.lib.BasicTupleStructure;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.lib.ExplicitTupleCollection;
import org.objectweb.medor.tuple.lib.MemoryTuple;

/**
 * In this example, we construct Fields, TupleStructure, TupleCollection and
 * Expressions...
 */
public class MedorExpressionExample {
    public static void main(String[] args)
			throws MedorException, ExpressionException {
        //Creating Fields
        Field[] fields = new Field[4];
        fields[0] = new BasicField("NSS", PTypeSpace.INT);
        fields[1] = new BasicField("NAME", PTypeSpace.STRING);
        fields[2] = new BasicField("ADRESS", PTypeSpace.STRING);
        fields[3] = new BasicField("SEX", PTypeSpace.CHAR);

        // Creating a TupleStructure Object
        TupleStructure ts = new BasicTupleStructure(fields);

        // Creating an arrays of VariableOperand
        Operand[] vops1 = new Operand[4];
        // Creating a simple variableOperand
        vops1[0] = new BasicOperand(201);
        vops1[1] = new BasicOperand("MEDOR");
        vops1[2] = new BasicOperand("MARS");
        vops1[3] = new BasicOperand('M');

        Operand[] vops2 = new Operand[4];
        vops2[0] = new BasicOperand(101);
        vops2[1] = new BasicOperand("JORM");
        vops2[2] = new BasicOperand("MARS");
        vops2[3] = new BasicOperand('M');

        // Creating Tuples
        Tuple t1 = new MemoryTuple(vops1);
        Tuple t2 = new MemoryTuple(vops2);

        // Creating an ExplicitTupleCollection
        ExplicitTupleCollection tc = new ExplicitTupleCollection(ts);

        //Inserting Tuples
        tc.insertTuple(t1);
        tc.insertTuple(t2);
        tc.first();
        tc.next();tc.next();tc.next();tc.previous();tc.previous();tc.previous();
        System.out.println(tc.getRow());
        //Displaying the content of the TupleCollection
        tc.display();

        // Creating an Expression = f(x,y) = ((x+13)-y)>74 + fieldOperand
        Operand value13 = new BasicOperand(13);
        Operand value74 = new BasicOperand(74);
        // Creating a FieldOperand
        FieldOperand fop = new BasicFieldOperand(fields[0]);
        fop.setIndex(1);
        ParameterOperand x = new BasicParameterOperand(PTypeSpace.INT, "x");
        ParameterOperand y = new BasicParameterOperand(PTypeSpace.INT, "y");
        BinaryOperator plus = new Plus();
        plus.setExpression(0, value13);
        plus.setExpression(1, x);
        BinaryOperator minus = new Minus();
        minus.setExpression(0, plus);
        minus.setExpression(1, y);
        ParameterOperand[] parameters = new BasicParameterOperand[2];
        parameters[0] = x;
        parameters[1] = y;
        x.setValue(12);
        y.setValue(99);
        Expression plus1 = new Plus(value74, fop);
        Comparator comp = new Greater(minus, plus1);
        comp.compileExpression();
        comp.evaluate(parameters, t1);
        System.out.println("The result of the expression (12+13)-99   :  " + minus.compileExpression().getInt());
        System.out.println("The result of the expression 74 + fop   :  " + plus1.compileExpression().getInt());
        System.out.println("The result of the expression ((12+13)-99)>74  :   " + comp.compileExpression().getBoolean());
        Substring subStr = new Substring(new BasicOperand("medorobjectwebforge"),minus,new BasicOperand(3));
        subStr.compileExpression();
        subStr.evaluate(parameters,null);
        System.out.println("The result of the expression substring(medorobjectwebforge,13,3) = " + subStr.compileExpression().getString());
    }
}
