/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.eval.api.QueryEvaluator;
import org.objectweb.medor.eval.lib.SelectEvaluator;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.VariableOperand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.BasicVariableOperand;
import org.objectweb.medor.expression.lib.Lower;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.lib.BasicField;
import org.objectweb.medor.lib.BasicTupleStructure;
import org.objectweb.medor.optim.api.ExecPlanGenerator;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.api.TCQueryLeaf;
import org.objectweb.medor.query.lib.MedorTCQueryLeaf;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.tuple.lib.ExplicitTupleCollection;
import org.objectweb.medor.tuple.lib.MemoryTuple;

/**
 * This is an example of a simple query over a TupleCollection Object data store...
 */
public class MedorTCQuery {
    public static void main(String[] args) throws MedorException{
        // Creating a TupleCollection QueryLeaf
        //Creating Fields
        Field[] fields = new Field[4];
        fields[0] = new BasicField("NSS", PTypeSpace.INT);
        fields[1] = new BasicField("NAME", PTypeSpace.STRING);
        fields[2] = new BasicField("ADRESS", PTypeSpace.STRING);
        fields[3] = new BasicField("SEX", PTypeSpace.CHAR);

        // Creating a TupleStructure
        TupleStructure ts = new BasicTupleStructure(fields);

        // Creating an arrays of VariableOperand
        VariableOperand[] vops1 = new VariableOperand[4];
        // Creating a simple variableOperand
        vops1[0] = new BasicVariableOperand(201);
        vops1[1] = new BasicVariableOperand("MEDOR");
        vops1[2] = new BasicVariableOperand("MARS");
        vops1[3] = new BasicVariableOperand('M');

        VariableOperand[] vops2 = new VariableOperand[4];
        vops2[0] = new BasicVariableOperand(101);
        vops2[1] = new BasicVariableOperand("JORM");
        vops2[2] = new BasicVariableOperand("MARS");
        vops2[3] = new BasicVariableOperand('M');

        // Creating Tuples
        Tuple t1 = new MemoryTuple(vops1);
        Tuple t2 = new MemoryTuple(vops2);

        // Creating an ExplicitTupleCollection
        ExplicitTupleCollection tc = new ExplicitTupleCollection(ts);

        //Inserting Tuples
        tc.insertTuple(t1);
        tc.insertTuple(t2);

        TCQueryLeaf medorQTLeaf = new MedorTCQueryLeaf("", "jorm_medor", tc);
        QueryTreeField mfield1 = medorQTLeaf.addField("NSS", PTypeSpace.INT, fields[0]);
        QueryTreeField mfield2 = medorQTLeaf.addField("NAME", PTypeSpace.STRING, fields[1]);
        QueryTreeField mfield3 = medorQTLeaf.addField("ADRESS", PTypeSpace.STRING, fields[2]);

        // The filter expression : a filtering....
        FieldOperand mfieldOp = new BasicFieldOperand(new BasicField("NSS", PTypeSpace.INT));
        BasicOperand mvalue = new BasicOperand(1000);
        Expression mFilter = new Lower(mfieldOp, mvalue);

        // Settign the filter
        medorQTLeaf.setQueryFilter(mFilter);

        // Preparing execution, generate indexes and setting TupleLoader
        ExecPlanGenerator indexesGen = new IndexesGenerator();

        System.out.print(" Generating indexes...");
		QueryTree query = null;
		try {
			query = indexesGen.transform(medorQTLeaf);
		}
		catch (Exception e) {
			System.out.println("Fail:");
			e.printStackTrace();
		}
        System.out.println("success");

        // Creates an evaluator associated to the QueryTree
        QueryEvaluator evaluator = new SelectEvaluator(query, 0);

        // Calculates and gets the required connection resources for this query
        ConnectionResources connRess = evaluator.getRequiredConnectionResources();

        // Launching Medor Evaluator...
        // No connections and no parameters...
        TupleCollection queryResult = evaluator.evaluate(null, null, null);

        // Browsing results
        System.out.println("RESULTS   :");
        int cpt = 0;
        if (!queryResult.isEmpty()) {
            do {
                cpt++;
                System.out.println("Tuple " + cpt + " :");
                System.out.println(queryResult.getMetaData().getField(1).getName() +
                        " = " + queryResult.getTuple().getObject(1));
                System.out.println(queryResult.getMetaData().getField(2).getName() +
                        " = " + queryResult.getTuple().getObject(2));
                System.out.println();
            } while (queryResult.next());
        } else {
            System.out.println("No result for this query  ");
        }
    }
}
