/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.BasicDataStore;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.eval.api.QueryEvaluator;
import org.objectweb.medor.eval.lib.SelectEvaluator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.Lower;
import org.objectweb.medor.expression.lib.Minus;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.optim.api.ExecPlanGenerator;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.medor.optim.lib.SourceCodeGeneratorCompiler;
import org.objectweb.medor.query.api.CalculatedField;
import org.objectweb.medor.query.api.PropagatedField;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbField;
import org.objectweb.medor.query.rdb.api.RdbStringQueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicQualifiedTable;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.tuple.api.TupleCollection;

import java.io.File;
import java.sql.Connection;
import java.sql.SQLException;

public class TestMedorCompiler {
    public static void main(String arg[])
			throws ExpressionException, MedorException, SQLException {
        //Creating a QueryTree...
        // THE URL OF THE DATA BASE
        String dbURL = new String("jdbc:idb:c:/EnhydraDB/Examples/sample.prp");
        // THE SQL STRING QUERY
        String sqlQuery1 = new String("SELECT * FROM Sample$db$Cols");
        // THE SQL STRING QUERY
        String sqlQuery2 = new String("SELECT * FROM Sample$db$Cols$pKey");

        DataStore ds = new BasicDataStore(DataStore.JDBC_STORE, "sample.prp");
        QualifiedTable table1 = new BasicQualifiedTable("sample$db$Cols", "sampledbCols");

        // We creates a QueryLeaf with an empty TupleStructure
        RdbStringQueryLeaf sqlLeaf = new BasicRdbStringQueryLeaf(ds, sqlQuery1, "");

        // Adding Fields that will be projected
        RdbField field1 = sqlLeaf.addRdbField("ColID", PTypeSpace.INT, "ColID");
        RdbField field2 = sqlLeaf.addRdbField("TableId", PTypeSpace.INT, "TableId");
        RdbField field3 = sqlLeaf.addRdbField("ColName", PTypeSpace.STRING, "ColName");


        // Creating a SelectProject Operation
        // At first the filter:
        FieldOperand fieldOp = new BasicFieldOperand(field1);
        BasicOperand value = new BasicOperand(1013);
        Expression filter = new Lower(fieldOp, value);
        // Then the Operation QueryNode
        SelectProject sp = new SelectProject(filter, "");

        // Adding Fields wich will be projected
        PropagatedField pf1 = sp.addPropagatedField("Col_Att1", PTypeSpace.INT, new QueryTreeField[]{field1});
        PropagatedField pf2 = sp.addPropagatedField("TableID_Att2", PTypeSpace.INT, new QueryTreeField[]{field2});
        PropagatedField pf3 = sp.addPropagatedField("ColName_Att3", PTypeSpace.STRING, new QueryTreeField[]{field3});

        // Adding CalculatedFields
        // The Arithmetic axpression of the first CalculatedField = Col_Att1 + TableID_Att2
        ParameterOperand x = new BasicParameterOperand(PTypeSpace.INT, "x");
        Expression exp1 = new Plus(new Plus(new BasicFieldOperand(field1), new BasicFieldOperand(field2)), x);
        // The Arithmetic axpression of the second CalculatedField = Col_Att1 - TableID_Att2
        Expression exp2 = new Minus(new BasicFieldOperand(field1), new BasicFieldOperand(field2));
        // Add The first calculedfield
        CalculatedField cf1 = sp.addCalculatedField("Col_Att1 + TableID_Att2 + x", PTypeSpace.INT, exp1);
        // Add the second CalculatedField
        CalculatedField cf2 = sp.addCalculatedField("Col_Att1 -TableID_Att2", PTypeSpace.INT, exp2);
        // This variable contains the path driver
        String idbdriver = "org.enhydra.instantdb.jdbc.idbDriver";
        JDBCConnection jdbcConn = new JDBCConnection();
        Connection connection = jdbcConn.getConnection(dbURL, idbdriver);

        // Preparing execution, generate indexes and setting TupleLoader
        ExecPlanGenerator indexesGen = new IndexesGenerator();

        System.out.print(" Generating indexes...");
        QueryTree query1 = indexesGen.transform(sp);
        System.out.println("success");


        String dir = "h:" + File.separator + "MedorRepositories" + File.separator + "Mars.Medor" +
            File.separator + "build" + File.separator + "org" + File.separator + "objectweb" +
            File.separator + "medor" + File.separator + "filter" + File.separator + "lib";
        String dir1 = "h:" + File.separator + "MedorRepositories" + File.separator + "Mars.Medor" + File.separator + "build";
        String dir2 = "h:" + File.separator + "MedorRepositories" + File.separator + "Mars.Medor" + File.separator + "src";

        SourceCodeGeneratorCompiler mComp = new SourceCodeGeneratorCompiler(dir, dir1, new String[]{dir2});
        // Adding Jorm classpath
        mComp.addLibraryPath("h:" + File.separator + "MedorRepositories" + File.separator + "Mars.Medor" + File.separator + "externals" + File.separator + "ow_jorm.jar");

        // Generating source code and load classes
        QueryTree query = mComp.transform(query1);

        // Creates an evaluator associated to the QueryTree
        QueryEvaluator evaluator = new SelectEvaluator(query, 0);

        // Calculates and gets the required connection ressources for this query
        ConnectionResources connRess = evaluator.getRequiredConnectionResources();

        // Gets the QueryLeafs that requierd connections
        QueryLeaf[] leafs = connRess.getRequiredQueryLeafConnection();

        // Setting QueryLeaf's appropriated connection Object
        for (int cpt = 0; (cpt < leafs.length); cpt++) {
            connRess.setConnection(leafs[cpt], connection);// In our simple case
        }

        // Setting Parameter Values
        x.setValue(-1800);

        // Launching Medor Evaluator...
        TupleCollection queryResult = null;
		//queryResult = evaluator.evaluate(new ParameterOperand[]{x}, connRess);

        // Browsing results
        System.out.println("RESULTS   :");
        int cpt = 0;

        if (!queryResult.isEmpty()) {
            do {
                cpt++;
                cpt++;
                System.out.println("Tuple " + cpt + " :");
                System.out.println(queryResult.getMetaData().getField(1).getName() +
                        " = " + queryResult.getTuple().getObject(1));
                System.out.println(queryResult.getMetaData().getField(2).getName() +
                        " = " + queryResult.getTuple().getObject(2));
                System.out.println(queryResult.getMetaData().getField(3).getName() +
                        " = " + queryResult.getTuple().getObject(3));
                System.out.println(queryResult.getMetaData().getField(4).getName() +
                        " = " + queryResult.getTuple().getObject(4));
                System.out.println();
            }
            while (queryResult.next());
        }
        else
            System.out.println("No result for this query");
        connection.close();
    }
}
