/**
 * Copyright (C) 2004-2005 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.medor.clone;

import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapterFactory;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.BasicDataStore;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.lib.And;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.Equal;
import org.objectweb.medor.expression.lib.Greater;
import org.objectweb.medor.expression.lib.Or;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.Count;
import org.objectweb.medor.lib.BasicField;
import org.objectweb.medor.lib.BasicTupleStructure;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.JoinProject;
import org.objectweb.medor.query.lib.MedorTCQueryLeaf;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.query.rdb.RdbQueryTreeComparator;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbExpField;
import org.objectweb.medor.query.rdb.api.RdbField;
import org.objectweb.medor.query.rdb.lib.AggregateRdbQueryNode;
import org.objectweb.medor.query.rdb.lib.BasicQualifiedTable;
import org.objectweb.medor.query.rdb.lib.BasicRdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.tuple.lib.ExplicitTupleCollection;
import org.objectweb.medor.tuple.lib.MemoryTuple;

/**
 * Test the cloning of QueryTree
 *
 * @author S. Chassande-Barrioz
 */
public class TestQueryTreeCloning extends TestQueryTreeHelper {
    static {
        registerComparator(new RdbQueryTreeComparator());
    }

	public TestQueryTreeCloning(String n) {
		super(n);
	}

	public void testRdbExpQL() throws Exception {
		QualifiedTable table = new BasicQualifiedTable("T_REF_SING_A", "T_REF_SING_A_0");
		String name = "org.objectweb.medor.optim.jorm.rdb.A";
		BasicRdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, new QualifiedTable[] {table}, name);
		ql.setDataStore(new BasicDataStore(DataStore.JDBC_STORE, "RDB"));
		ql.setDistinct(true);
		try {
			ql.setRdbAdapter(RdbAdapterFactory.getTypeConverter((String) null));
		} catch (RdbAdapterException ex) {
			fail(ex.getMessage());
		}
		String prefix = name + ".";
		ql.addRdbField(prefix + "id", PTypeSpace.LONG, "id_col", table);
		ql.addRdbField(prefix + "field1", PTypeSpace.STRING, "FIELD_1", table);
		ql.addRdbField(prefix + "field2", PTypeSpace.STRING, "FIELD_2", table);

		Object clone = ql.clone();
		assertNotNull("Null clone", clone);
		assertEquals("Bad class", ql.getClass(), clone.getClass());
		equals("testRdbExpQL ", ql, (QueryTree) clone, logger);
	}

	public void testRdbExpQLWithFilter() throws Exception {
		QualifiedTable table = new BasicQualifiedTable("T_REF_SING_A", "T_REF_SING_A_0");
		String name = "org.objectweb.medor.optim.jorm.rdb.A";
		BasicRdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(
			new BasicDataStore(DataStore.JDBC_STORE, "RDB"),
			new QualifiedTable[] {table},
			name);
		ql.setDistinct(true);
		try {
			ql.setRdbAdapter(RdbAdapterFactory.getTypeConverter((String) null));
		} catch (RdbAdapterException ex) {
			fail(ex.getMessage());
		}
		String prefix = name + ".";
		ql.addRdbField(prefix + "id", PTypeSpace.LONG, "id_col", table);
		RdbExpField f1 = ql.addRdbField(prefix + "field1", PTypeSpace.STRING, "FIELD_1", table);
		RdbExpField f2 = ql.addRdbField(prefix + "field2", PTypeSpace.STRING, "FIELD_2", table);

		ql.setQueryFilter(new Or(
			new Greater(new BasicFieldOperand(f1), new BasicOperand(12)),
			new And(
				new BasicOperand(true),
				new Equal(new BasicFieldOperand(f2), new BasicOperand(12))
			)));

		Object clone = ql.clone();
		assertNotNull("Null clone", clone);
		assertEquals("Bad class", ql.getClass(), clone.getClass());
		equals("testRdbExpQLWithFilter ", ql, (QueryTree) clone, logger);

		//Check that field referenced in the filter are the same instance
		// than the one in the projected fields
		BasicRdbExpQueryLeaf cql = (BasicRdbExpQueryLeaf) clone;
		RdbExpField cf1 = (RdbExpField) cql.getField(prefix + "field1");
		RdbExpField cf2 = (RdbExpField) cql.getField(prefix + "field2");
		assertTrue("field f1 is not cloned !", f1 != cf1);
		assertTrue("field f2 is not cloned !", f2 != cf2);
		assertTrue("adapter cloned !", ql.getRdbAdapter() == cql.getRdbAdapter());
		assertEquals("bad adapter name !", ql.getRdbAdapterName(), cql.getRdbAdapterName());
		Operator op = (Operator) cql.getQueryFilter();
		BasicFieldOperand bfo = (BasicFieldOperand)
			((Operator) op.getExpression(0)).getExpression(0);
		assertTrue("field f1 from filter is not a field f1 from QT", bfo.getField() == cf1);
		bfo = (BasicFieldOperand)
			((Operator) ((Operator) op.getExpression(1)).getExpression(1)).getExpression(0);
		assertTrue("field f1 from filter is not a field f1 from QT", bfo.getField() == cf2);
	}

	public void testRdbStringQL() throws Exception {
		String name = "org.objectweb.medor.optim.jorm.rdb.A";
		BasicRdbStringQueryLeaf ql = new BasicRdbStringQueryLeaf(
			new BasicDataStore(DataStore.JDBC_STORE, "RDB"),
			"SELECT id_col, FIELD_1, FIELD_2 FROM TA wHERE id_col > 0",
			name);
		String prefix = name + ".";
		ql.addRdbField(prefix + "id", PTypeSpace.LONG, "id_col");
		ql.addRdbField(prefix + "field1", PTypeSpace.STRING, "FIELD_1");
		ql.addRdbField(prefix + "field2", PTypeSpace.STRING, "FIELD_2");

		Object clone = ql.clone();
		assertNotNull("Null clone", clone);
		assertEquals("Bad class", ql.getClass(), clone.getClass());
		equals("testRdbStringQL ", ql, (QueryTree) clone, logger);
	}

	public void testAggregateRdbQueryNode() throws Exception {
		QualifiedTable table = new BasicQualifiedTable("T_REF_SING_A", "T_REF_SING_A_0");
		String name = "org.objectweb.medor.optim.jorm.rdb.A";
		BasicRdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(
			new BasicDataStore(DataStore.JDBC_STORE, "RDB"),
			new QualifiedTable[] {table},
			name);
		String prefix = name + ".";
		ql.addRdbField(prefix + "id", PTypeSpace.LONG, "id_col", table);
		RdbExpField f1 = ql.addRdbField(prefix + "f1", PTypeSpace.STRING, "FIELD_1", table);
		RdbExpField f2 = ql.addRdbField(prefix + "f2", PTypeSpace.STRING, "FIELD_2", table);

		ql.setQueryFilter(new Or(
			new Greater(new BasicFieldOperand(f1), new BasicOperand(12)),
			new And(
				new BasicOperand(true),
				new Equal(new BasicFieldOperand(f2), new BasicOperand(12))
			)));

		QueryTreeField expF2 = (QueryTreeField)
			ql.getTupleStructure().getField(prefix + "f2");
		QueryTreeField expF1 = (QueryTreeField)
			ql.getTupleStructure().getField(prefix + "f1");
		ql.setDistinct(true);
		ql.setIsSubquery(true);
		AggregateRdbQueryNode expected = new AggregateRdbQueryNode(
			new QueryTreeField[]{expF1, expF2}, "grouped",
			new QueryTreeField[] {}, ql, null, "nestNode");
		Count theCount = new Count(new BasicFieldOperand(expF2), true);
		theCount.setCountAll();
		expected.addCalculatedField("count",
		        PTypeSpace.INT,
				theCount
				);

		Object clone = expected.clone();
		assertNotNull("Null clone", clone);
		assertEquals("Bad class", expected.getClass(), clone.getClass());
		equals("testAggregateRdbQueryNode ", expected, (QueryTree) clone, logger);
	}

	public void testMedorTCQueryLeaf() throws Exception {
		BasicTupleStructure ts = new BasicTupleStructure(new BasicField[] {
			new BasicField("f1", PTypeSpace.INT),
			new BasicField("f2", PTypeSpace.STRING),
			new BasicField("f3", PTypeSpace.LONG),
		});
		ExplicitTupleCollection tc = new ExplicitTupleCollection(ts);
		tc.insertTuple(new MemoryTuple(new Operand[]{
			new BasicOperand(1),
			new BasicOperand("a"),
			new BasicOperand(1l),
		}));
		tc.insertTuple(new MemoryTuple(new Operand[]{
			new BasicOperand(2),
			new BasicOperand("b"),
			new BasicOperand(2l),
		}));
		tc.insertTuple(new MemoryTuple(new Operand[]{
			new BasicOperand(3),
			new BasicOperand("c"),
			new BasicOperand(3l),
		}));
		MedorTCQueryLeaf ql = new MedorTCQueryLeaf(
			"toto, ", "TC_NAME", new ExplicitTupleCollection(ts));

		Object clone = ql.clone();
		assertNotNull("Null clone", clone);
		assertEquals("Bad class", ql.getClass(), clone.getClass());
		equals("testMedorTCQueryLeaf ", ql, (QueryTree) clone, logger);
	}

	public void testJoin() throws Exception {
		String aname = "org.objectweb.medor.optim.jorm.rdb.A";
		BasicRdbStringQueryLeaf aql = new BasicRdbStringQueryLeaf(
			new BasicDataStore(DataStore.JDBC_STORE, "RDB"),
			"SELECT AID, AF1, AF2 FROM TA",
			aname);
		String aprefix = aname + ".";
		RdbField aid = aql.addRdbField(aprefix + "aid", PTypeSpace.LONG, "AID");
		RdbField af1 = aql.addRdbField(aprefix + "af1", PTypeSpace.STRING, "AF1");
		RdbField af2 = aql.addRdbField(aprefix + "af2", PTypeSpace.STRING, "AF2");

		String bname = "org.objectweb.medor.optim.jorm.rdb.A";
		BasicRdbStringQueryLeaf bql = new BasicRdbStringQueryLeaf(
			new BasicDataStore(DataStore.JDBC_STORE, "RDB"),
			"SELECT BID, BF1, BF2 FROM TB",
			bname);
		String prefix = bname + ".";
		RdbField bid = bql.addRdbField(prefix + "bid", PTypeSpace.LONG, "BID");
		RdbField bf1 = bql.addRdbField(prefix + "bf1", PTypeSpace.STRING, "BF1");
		RdbField bf2 = bql.addRdbField(prefix + "bf2", PTypeSpace.STRING, "BF2");

		JoinProject jp = new JoinProject(
			new Equal(new BasicFieldOperand(af1), new BasicFieldOperand(bf2)),
			"J1");
		jp.addPropagatedField(aid.getName(), aid.getType(), new QueryTreeField[]{aid});
		QueryTreeField ff1 = jp.addPropagatedField(af1.getName(), af1.getType(), new QueryTreeField[]{af1});
		jp.addPropagatedField(af2.getName(), af2.getType(), new QueryTreeField[]{af2});
		jp.addPropagatedField(bid.getName(), bid.getType(), new QueryTreeField[]{bid});
		jp.addPropagatedField(bf1.getName(), bf1.getType(), new QueryTreeField[]{bf1});

		SelectProject sp = new SelectProject("SP1");
		sp.setQueryFilter(new Equal(new BasicFieldOperand(ff1), new BasicOperand("azerty")));
		
		Object clone = sp.clone();
		
		assertNotNull("Null clone", clone);
		assertEquals("Bad class", sp.getClass(), clone.getClass());
		equals("testJoin ", sp, (QueryTree) clone, logger);
	}
}
