/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.medor.query;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.util.monolog.api.Logger;

/**
 * Defines a comparator of query tree based on Junit assertion.
 *
 * @author S.Chassande-Barrioz
 */
public interface QueryTreeComparator {
    /**
     * compares two QueryTree instances. The comparaison has to be based on 
     * junit assertion (ex: Assert.assertEquals(...)). 
     * @param prefix is a string to put before each logging trace
     * @param qt1 is the first querytree to compare
     * @param qt2 is the second querytree to compare
     * @param log is the logger to use
     * @return true only if the  specified query tree instances has been 
     * compared. The boolean value does not indicate if qts are equals but only
     * if they have been compared or not. 
     * @throws MedorException
     */
    boolean assertEquals(String prefix, QueryTree qt1, QueryTree qt2,
            Logger log) throws MedorException;
    
    /**
     * compares two Field instances. The comparaison has to be based on 
     * junit assertion (ex: Assert.assertEquals(...)). 
     * @param prefix is a string to put before each logging trace
     * @param qt1 is the first field to compare
     * @param qt2 is the second field to compare
     * @param log is the logger to use
     * @return true only if the  specified field instances has been 
     * compared. The boolean value does not indicate if field are equals but only
     * if they have been compared or not. 
     * @throws MedorException
     */
    boolean assertEquals(String prefix, Field f1, Field f2, Logger log)
        throws MedorException;
}
