/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.query;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.lib.And;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.Greater;
import org.objectweb.medor.expression.lib.Lower;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbExpField;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicQualifiedTable;
import org.objectweb.medor.query.rdb.lib.BasicRdbExpQueryLeaf;

/**
 * @author	<A HREF="mailto:alia.mourad@rd.francetelecom.com><b>
 *			Mourad Alia
 *			</b></A>
 *		<A HREF="mailto:alexandre.lefebvre@rd.francetelecom.com><b>
 * 			Alexandre Lefebvre
 </b></A>
 */
public class RdbExpLeaf {

    public static void main(String[] arg) {
        try {
            //create qualified tables
            QualifiedTable t1 = new BasicQualifiedTable("t1", null);
            QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
            QualifiedTable t3 = new BasicQualifiedTable("t3", "al3");
            QualifiedTable[] tables = {t1, t2};
            //create a new leaf
            RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
            //add fields
            RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
            RdbExpField f2 = ql.addRdbField("f2", null, "col2", t2);
            //create a filter to be added to the query leaf
            FieldOperand fieldOp1 = new BasicFieldOperand(f1);
            FieldOperand fieldOp2 = new BasicFieldOperand(f2);
            BasicOperand valueL = new BasicOperand(1013);
            BasicOperand valueR = new BasicOperand("coucou");
            Expression left = new Lower(fieldOp1, valueL);
            Expression right = new Greater(fieldOp2, valueR);
            Expression filter = new And(left, right);
            //assigne the filter to the query leaf
            ql.setQueryFilter(filter);
            System.out.println("First request");
            //computes the SQL query
            System.out.println(ql.getSqlRequest(null, false, false));

            ParameterOperand fieldOp3 =
                new BasicParameterOperand(PTypeSpace.STRING, "x");
            Expression left2 = new Lower(fieldOp3, valueL);
            filter = new And(left2, right);
            ql.setQueryFilter(filter);
            ql.addGroupBy(f2);
            System.out.println(ql.getSqlRequest(null, false, false));
        }
        catch (Exception e) {
            System.err.println("Exit with exception: " + e.getMessage());
            do {
                e.printStackTrace();
                if (e instanceof PException)
                    e = ((PException) e).getNestedException();
                else
                    e = null;
            }
            while (e != null);
        }
    }
}
