/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.medor.query.rdb;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.query.QueryTreeComparator;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbExpField;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.api.RdbField;
import org.objectweb.medor.query.rdb.api.RdbStringQueryLeaf;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import junit.framework.Assert;

/**
 * Is a helper for comparing query tree containing Rdb stuffs.
 *
 * @author S.Chassande-Barrioz
 */
public class RdbQueryTreeComparator 
    implements QueryTreeComparator {

    public RdbQueryTreeComparator() {
        super();
    }

    public boolean assertEquals(String prefix, QueryTree qt1, QueryTree qt2,
            Logger log) throws MedorException {
        if (qt1 instanceof RdbExpQueryLeaf) {
            RdbExpQueryLeaf ql1 = (RdbExpQueryLeaf) qt1;
            RdbExpQueryLeaf ql2 = (RdbExpQueryLeaf) qt2;
            QualifiedTable[] t1 = ql1.getQualifiedTables();
            QualifiedTable[] t2 = ql2.getQualifiedTables();
            Assert.assertEquals(prefix + "RdbExpQueryLeaf(" + qt1.getName() + "): Number of QualifiedTable is different",
                t1.length, t2.length);
            for (int i = 0; i < t1.length; i++) {
                Assert.assertEquals(prefix + "RdbExpQueryLeaf(" + qt1.getName() + "): Not same table name",
                    t1[i].getTableName(), t2[i].getTableName());
                Assert.assertEquals(prefix + "RdbExpQueryLeaf(" + qt1.getName() + "): Not same alias name",
                    t1[i].getAliasName(), t2[i].getAliasName());
            }
            try {
                log.log(BasicLevel.DEBUG, "SQL for leaf 2"
                        + ql2.getSqlRequest(null, false, false));
                log.log(BasicLevel.DEBUG, "SQL for leaf 1"
                        + ql1.getSqlRequest(null, false, false));
                Assert.assertEquals(prefix + "Not same SQL request",
                        ql1.getSqlRequest(null, false, false),
                        ql2.getSqlRequest(null, false, false));
            }
            catch (ExpressionException e) {
                throw new MedorException(e);
            }
            return true;
            
        } else if (qt1 instanceof RdbStringQueryLeaf) {
            RdbStringQueryLeaf ql1 = (RdbStringQueryLeaf) qt1;
            RdbStringQueryLeaf ql2 = (RdbStringQueryLeaf) qt2;
            try {
                Assert.assertEquals(prefix + "RdbStringQueryLeaf(" + qt1.getName() + "): Not same request",
                    ql1.getSqlRequest(null, false, false),
                        ql2.getSqlRequest(null, false, false));
            }
            catch (ExpressionException e) {
                log.log(BasicLevel.ERROR,
                    prefix + "RdbStringQueryLeaf(" + qt1.getName() + "): Impossible to fetch the sqlRequest: ",
                    e);
                throw new MedorException(e);
            }
            catch (MedorException e) {
                log.log(BasicLevel.ERROR,
                    prefix + "RdbStringQueryLeaf(" + qt1.getName() + "): Impossible to fetch the sqlRequest: ",
                    e);
                throw e;
            }
            return true;
        }         
        return false;
    }
    
    public boolean assertEquals(String prefix, Field f1, Field f2, Logger log)
        throws MedorException {
        if (f1 instanceof RdbExpField) {
            RdbExpField ref1 = (RdbExpField) f1;
            RdbExpField ref2 = (RdbExpField) f2;
            Assert.assertEquals("RdbExpField: Not same column name",
                ref1.getColumnName(), ref2.getColumnName());
            Assert.assertEquals("RdbExpField: Not same table name",
                ref1.getTable().getTableName(), ref2.getTable().getTableName());
            Assert.assertEquals("RdbExpField: Not same table name",
                ref1.getTable().getAliasName(), ref2.getTable().getAliasName());
            return true;
        } else if (f1 instanceof RdbField) {
            RdbField ref1 = (RdbField) f1;
            RdbField ref2 = (RdbField) f2;
            Assert.assertEquals("RdbField: Not same column name",
                ref1.getColumnName(), ref2.getColumnName());
            return true;
        }
        return false;
    }    
}
